/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
 /* $Id: callbacks.c,v 1.18 2002/10/20 01:09:47 dolphy Exp $ */
 
#include "getcontrol.h"

/* Callbacks */

gboolean
getcontrol_quit_callback (GtkWidget * widget, gpointer data)
{	
	remove_all_timers ();
	
	if (frame) /* Freeing all our pixbufs */
		gdk_pixbuf_unref (frame);
	if (frame_backup)
		gdk_pixbuf_unref (frame_backup);
	if (background)
		gdk_pixbuf_unref (background);
	if (background_orig)
		gdk_pixbuf_unref (background_orig);
	
	if (main_menu_images_list)
		free_images_list (main_menu_images_list);
	
	if (prefs_menu_images_list)
		free_images_list (prefs_menu_images_list);
		
	if (themes_menu_images_list)
		free_images_list (themes_menu_images_list);
	
	if (screensaver_menu_images_list)
		free_images_list (screensaver_menu_images_list);
		
	if (anim_menu_images_list)
		free_images_list (anim_menu_images_list);
	
	if (current_theme) g_free(current_theme);
	
	if (selected_label) g_free(selected_label);
	
	if (font_markup) g_free (font_markup);
	
	gtk_main_quit ();
	
}


gboolean
getcontrol_about_callback (GtkWidget * widget, gpointer data)
{
	GdkPixbuf *pixbuf = NULL;
	static GtkWidget *about = NULL;
	const gchar *authors[] = {
		"Julien MOUTTE. <julien@moutte.net>",
		"Matthias SAOU. <matthias.saou@est.une.marmotte.net>",
		NULL
	};

	/* Documenters credits */

	gchar *documenters[] = {
		NULL
	};

	/* Translator credits */

	gchar *translator_credits = _("translator_credits");
	
	if (about) {
		gtk_window_present (GTK_WINDOW (about));
		return;
	}

	{
		char *filename = NULL;
		filename = gnome_program_locate_file (
				NULL,
				GNOME_FILE_DOMAIN_APP_PIXMAP,
				("getcontrol.png"),
				TRUE, NULL);

		if (filename != NULL)
		{
			pixbuf = gdk_pixbuf_new_from_file(filename, NULL);
			g_free (filename);
		}
	}

	about = gnome_about_new (
			_("Getcontrol"), VERSION,
			"(C) 2002 the Free Software Foundation",
			_("The front end you need for multimedia boxes."),
			(const char **)authors,
			(const char **)documenters,
			strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
			pixbuf
				);

	gtk_window_set_transient_for (GTK_WINDOW (about), GTK_WINDOW (app));
	gtk_widget_show (about);
	g_signal_connect (
			G_OBJECT (about), "destroy",
			G_CALLBACK (gtk_widget_destroyed), &about);

	return TRUE;
}

/* Expose callback for the drawing area */
gboolean
expose_callback (GtkWidget * widget, GdkEventExpose * event, gpointer data)
{
	guchar *pixels;
	int rowstride;
	guint text_xpos = 0, text_ypos = 0;
	PangoLayout * getcontrol_plyt = NULL;
	PangoAttrList * getcontrol_attrlist = NULL;
	gchar * parsed_label = NULL, * marked_up_label = NULL;
	
	rowstride = gdk_pixbuf_get_rowstride (frame);

	pixels = gdk_pixbuf_get_pixels (frame) + rowstride * event->area.y + event->area.x * 3;

	gdk_draw_rgb_image_dithalign (
			widget->window,
			widget->style->black_gc,
			event->area.x, event->area.y,
			event->area.width, event->area.height,
			GDK_RGB_DITHER_NORMAL,
			pixels, rowstride,
			event->area.x, event->area.y);
	
	if (current_status == SELECTED) {
		
		marked_up_label = g_strdup_printf ("%s%s</span>",
							font_markup, selected_label);
		
		pango_parse_markup (
			marked_up_label, -1, 0,
			&getcontrol_attrlist,
			&parsed_label,
			NULL, NULL);
		
		if (marked_up_label) g_free (marked_up_label);
	
		getcontrol_plyt = gtk_widget_create_pango_layout (da, parsed_label);
		
		pango_layout_set_attributes (getcontrol_plyt, getcontrol_attrlist);
		
		pango_layout_get_pixel_size (getcontrol_plyt, &text_xpos, &text_ypos);
	
		text_xpos = xmid - (text_xpos / 2);

		text_ypos = (ymid * 2) - (text_ypos);
	
		gdk_draw_layout (
			widget->window,
			widget->style->black_gc,
			text_xpos, text_ypos,
			getcontrol_plyt);
			
		if (parsed_label)
			g_free (parsed_label);
	}
		
	return TRUE;
}

/* Callback for key_press_event management */
gboolean
key_pressed_callback (GtkWidget * widget, GdkEventKey * event, image_node * images_list)
{
	image_node * l_images_list = NULL;
	
	switch (current_status) {
		case IDLE :	
			
			remove_all_timers ();
			
			switch (event->keyval) {
				case 65363 :
					direction = RIGHT;
					break;;
				case 65361 :
					direction = LEFT;
					break;;
				case 65293 :
					l_images_list = current_menu_images_list;
					while ( !(l_images_list->selected) && (l_images_list) )
						l_images_list = l_images_list->next_image;
					l_images_list->image_link (l_images_list->data);
					return TRUE;		
				case 32 :
					if (fullscreen)
						fullscreen = FALSE;
					else
						fullscreen = TRUE;
					
					/* Storing the change in GConf */
	
					gconf_client_set_bool (conf_client,
							FULLSCREEN_PATH, fullscreen, NULL);
					
					return TRUE;
				case 109 :
					activate_main_menu (current_menu_images_list);
					return TRUE;
				default :
					return TRUE;;
			}
	
			menu_rotation_manager (current_menu_images_list);
			
			break;;
		case SELECTED : /* Key pressed while selected checking if we validate or
						not and disabling screen saver */

			remove_all_timers ();
		
			switch (event->keyval) {
				case 65363 :
					direction = RIGHT;
					break;;
				case 65361 :
					direction = LEFT;
					break;;
				case 32 :
					if (fullscreen)
						fullscreen = FALSE;
					else
						fullscreen = TRUE;
					
					/* Storing the change in GConf */
	
					gconf_client_set_bool (conf_client,
							FULLSCREEN_PATH, fullscreen, NULL);
					
					return TRUE;
				case 65293 :
					l_images_list = current_menu_images_list;
					while ( !(l_images_list->selected) && (l_images_list) )
						l_images_list = l_images_list->next_image;
					l_images_list->image_link (l_images_list->data);
					return TRUE;
				case 109 :
					activate_main_menu (current_menu_images_list);
					return TRUE;
				default :
					return TRUE;
			}
			
			menu_rotation_manager (current_menu_images_list);
			
			break;;
		case SCREEN_SAVING :
			
			remove_all_timers ();
			
			draw_menu(current_menu_images_list);
			
			break;;
			
		case ROTATING :
		case DESELECTING :
			switch (event->keyval) {
				case 65363 :
					stack_order (&order_queue, RIGHT, ROTATE_ORDER);
					return TRUE;
				case 65361 :
					stack_order (&order_queue, LEFT, ROTATE_ORDER);
					return TRUE;
				case 65293 :
					stack_order (&order_queue, 0, ACTIVATE_ORDER);
				default :
					return TRUE;
			}
			
			break;;
	}

	
	return TRUE;
}

/* Callback for resizing and first drawing */
gboolean
configure_callback (GtkWidget * widget, GdkEventConfigure *event)
{

	gdouble width_ratio_scale, height_ratio_scale;
	
	remove_all_timers ();
	
	if (frame) /* Freeing all our pixbufs */
		gdk_pixbuf_unref (frame);
	if (frame_backup)
		gdk_pixbuf_unref (frame_backup);
	if (background)
		gdk_pixbuf_unref (background);
	
	draw_width = widget->allocation.width;
	draw_height = widget->allocation.height;

	/* Creating our pixbufs */
	
	frame = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
							FALSE, 8, draw_width,
							draw_height);
		
	frame_backup = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
							FALSE, 8, draw_width,
							draw_height);

	background = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
							FALSE, 8, draw_width,
							draw_height);
	
	/* Middle of the picture */
	
	xmid = draw_width / 2.0;
	ymid = draw_height / 2.0;

	/* Calculating radius */

	radius = MIN (xmid, ymid) / 2.0;
	
	width_ratio_scale = (gdouble) draw_width / (gdouble) back_width;
	height_ratio_scale = (gdouble) draw_height / (gdouble) back_height;
	
	/* Drawing & scaling our background */
	
	gdk_pixbuf_composite (
		background_orig,
		background,
		0, 0,
		draw_width, draw_height,
		0, 0,
		width_ratio_scale, height_ratio_scale,
		GDK_INTERP_BILINEAR,
		255);

	/* Drawing menu */
	
	draw_menu (current_menu_images_list);
	
	return TRUE;
}

gboolean
remove_all_timers ()
{
	if (select_timeout_id) /* Removing selection timer */
		gtk_timeout_remove (select_timeout_id);
	
	select_timeout_id = 0;
	
	if (screen_saver_timeout_id) /* Removing screen saver timer */
		gtk_timeout_remove (screen_saver_timeout_id);
	
	screen_saver_timeout_id = 0;
		
	if (draw_timeout_id) /* Removing animation timer */
		gtk_timeout_remove (draw_timeout_id);
	
	draw_timeout_id = 0;
	
	if (ss_draw_timeout_id)
		gtk_timeout_remove (ss_draw_timeout_id);
	
	ss_draw_timeout_id = 0;
	
	return TRUE;
}


/* This is called when our timer keys are changed. */
void
gconf_notify_global (GConfClient *client,
		   guint        cnxn_id,
		   GConfEntry  *entry,
		   gpointer     user_data)
{
		load_config ();
}

