/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
 /* $Id: draw.c,v 1.9 2002/10/16 13:07:09 dolphy Exp $ */
 
#include "getcontrol.h"

/* Background image and its dimensions */

GdkPixbuf * background = NULL, * background_orig = NULL;

/* Current frame and backup frame */

GdkPixbuf * frame = NULL, * frame_backup = NULL;

/* Fullscreen setting */

gboolean fullscreen = FALSE;

/* ===========Images list draw============ */

/* draw_images
	Function that runs in the images table and draw visible ones
	to their stored coordinates with stored alpha level and zoom
	ratio.	
*/

gboolean
draw_images (image_node * images_list)
{
	image_node * draw_images_list = NULL;
	
	/* Copying the head pointer of the images list */
	
	draw_images_list = images_list;
	
	if (!draw_images_list) /* Images list not initialized */
		return FALSE;
	
	while (draw_images_list) {
		
		GdkRectangle r1, r2, dest;
		
		/*g_message ("image %d infos : angle %d xpos %d ypos %d alpha %d zoom %d visible %d",
			i, images_coord[i][0], images_coord[i][1], images_coord[i][2],
			images_coord[i][3], images_coord[i][4], images_coord[i][5]);*/
		
		if (draw_images_list->visible) { /* If image is visible */
			
			r1.x = draw_images_list->xpos;
			r1.y = draw_images_list->ypos;
			r1.width = draw_images_list->width * draw_images_list->zoom_ratio;
			r1.height = draw_images_list->height * draw_images_list->zoom_ratio;
			
			r2.x = 0;
			r2.y = 0;
			r2.width = draw_width;
			r2.height = draw_height;
		
			if (gdk_rectangle_intersect (&r1, &r2, &dest)) {
			
				gdk_pixbuf_composite (
					draw_images_list->image,
					frame,
					dest.x, dest.y,
					dest.width, dest.height,
					draw_images_list->xpos, draw_images_list->ypos,
					draw_images_list->zoom_ratio, draw_images_list->zoom_ratio,
					GDK_INTERP_BILINEAR,
					draw_images_list->alpha_level);
				
			}
			
		}
		
		draw_images_list = draw_images_list->next_image;
		
	}
	
	gtk_widget_draw (da, NULL);

	return TRUE;
}

/* ========================================= */

/* ===========Images list tools============= */

/* add_image_at_the_end
	Function adding an image_node at the end of an image_node list
*/

gboolean
add_image_at_the_end (image_node * new_image, image_node ** images_list)
{
	image_node * draw_images_list = NULL;

	/* Copying the head pointer of the images list */
	
	draw_images_list = *images_list;
	
	if (!draw_images_list) { /* Images list empty */
		*images_list = new_image;
		return TRUE;
	}
	
	while (draw_images_list->next_image)
		draw_images_list = draw_images_list->next_image;
	
	draw_images_list->next_image = new_image;
	
	return TRUE;
}

/* select_previous
	Function which moves the select flag on the previous node
*/

gboolean
select_previous (image_node * images_list)
{
	image_node * draw_images_list;
	
	draw_images_list = images_list;
	
	
	if (draw_images_list->selected) { /* First element selected */
		
		draw_images_list->selected = FALSE;
		
		while (draw_images_list->next_image) /* Jumping to last one */
			draw_images_list = draw_images_list->next_image;
		
		draw_images_list->selected = TRUE;
		
	}
	else {
	
		while (draw_images_list) {
			image_node * previous_node;
				
			previous_node = draw_images_list;
				
			draw_images_list = draw_images_list->next_image;
			
			if (!draw_images_list) /* End of the list reached */
				return FALSE;
			
			if (draw_images_list->selected) {
				draw_images_list->selected = FALSE;
				previous_node->selected = TRUE;
				draw_images_list = NULL;
			}
		}
	}
		
	return TRUE;
}

/* select_next
	Function which moves the select flag on the previous node
*/

gboolean
select_next (image_node * images_list)
{
	image_node * draw_images_list;
	
	draw_images_list = images_list;
	
	while (draw_images_list) {
		image_node * previous_node;
				
		previous_node = draw_images_list;
				
		draw_images_list = draw_images_list->next_image;
			
		if (previous_node->selected) {
			if (draw_images_list) {
				draw_images_list->selected = TRUE;
				previous_node->selected = FALSE;
				draw_images_list = NULL;
			}
			else {
				previous_node->selected = FALSE;
				images_list->selected = TRUE;
			}
		}
	
	}
		
	return TRUE;
}

/* dump_images_list
	Function which will take care of dumping an image_node list
	to stdout
*/

gboolean
dump_images_list (image_node * images_list)
{
	image_node * draw_images_list = NULL;
	gint i = 0;

	/* Copying the head pointer of the images list */
	
	draw_images_list = images_list;
	
	if (!draw_images_list) { /* Images list not initialized */
		g_message ("no images to dump");
		return FALSE;
	}
	
	while (draw_images_list) {
		g_message ("image number %d", i);
		g_message ("image file %s", draw_images_list->image_path);
		g_message ("image label %s", draw_images_list->image_label);
		g_message ("image angle %d", draw_images_list->angle);
		g_message ("image xpos %d", draw_images_list->xpos);
		g_message ("image ypos %d", draw_images_list->ypos);
		g_message ("image width %d", draw_images_list->width);
		g_message ("image height %d", draw_images_list->height);
		g_message ("image alpha_level %d", draw_images_list->alpha_level);
		g_message ("image zoom_ratio %f", draw_images_list->zoom_ratio);
		g_message ("image visible %d", draw_images_list->visible);
		g_message ("image selected %d", draw_images_list->selected);
		draw_images_list = draw_images_list->next_image;
		i++;
	}
	
	return TRUE;
}

/* free_images_list
	Function which will take care of freeing an image_node list
*/

gboolean
free_images_list (image_node * images_list)
{
	image_node * draw_images_list = NULL, * backup = NULL;

	/* Copying the head pointer of the images list */
	
	draw_images_list = images_list;
	
	if (!draw_images_list) { /* Images list not initialized */
		g_message ("no images to free");
		return FALSE;
	}
	
	while (draw_images_list) {
		
		g_object_unref (draw_images_list->image);
		if (draw_images_list->image_path)
			g_free (draw_images_list->image_path);
		if (draw_images_list->data)
			g_free(draw_images_list->data);
		if (draw_images_list->image_label)
			g_free(draw_images_list->image_label);
		
		backup = draw_images_list;
		
		draw_images_list = draw_images_list->next_image;
		
		if (backup)
			free (backup);
		
		backup=NULL;
	}
	
	return TRUE;
}

/* ========================================= */

/* ===========Images list tools============= */

/* wmspec_change_state
	Function that send Xevent to the window manager
	setting fullscreen mode or not
*/

void
wmspec_change_state (gboolean   add,
                     GdkWindow *window,
                     GdkAtom    state1,
                     GdkAtom    state2)
{
  XEvent xev;

#define _NET_WM_STATE_REMOVE        0    /* remove/unset property */
#define _NET_WM_STATE_ADD           1    /* add/set property */
#define _NET_WM_STATE_TOGGLE        2    /* toggle property  */  
  
  xev.xclient.type = ClientMessage; /* <- nok */
  xev.xclient.serial = 0; /* <- ok */
  xev.xclient.send_event = True; /* <- ok */
  xev.xclient.display = gdk_display; /* <- nok */
  xev.xclient.window = GDK_WINDOW_XID (window); /* <- ok */
  xev.xclient.message_type = gdk_x11_get_xatom_by_name ("_NET_WM_STATE"); /* <- ok */
  xev.xclient.format = 32; /* <- ok */
  xev.xclient.data.l[0] = add ? _NET_WM_STATE_ADD : _NET_WM_STATE_REMOVE; /* <- ok */
  xev.xclient.data.l[1] = gdk_x11_atom_to_xatom (state1); /* <- ok */
  xev.xclient.data.l[2] = gdk_x11_atom_to_xatom (state2); /* <-ok */
  
  XSendEvent (gdk_display, GDK_WINDOW_XID (gdk_get_default_root_window ()),
              False,
	      SubstructureRedirectMask | SubstructureNotifyMask,
	      &xev);
}

/* toggle_fullscreen
	Function which enable or disable fullscreen mode
	according to the fullscreen global flag
*/

gboolean
toggle_fullscreen (void)
{

	if (fullscreen) {
		gtk_widget_hide (app->statusbar);
		gtk_widget_hide (app->menubar);
		wmspec_change_state (TRUE,
			GTK_WIDGET (app)->window,
			gdk_atom_intern ("_NET_WM_STATE_FULLSCREEN", FALSE),
			GDK_NONE);
	}
	else {
		
		wmspec_change_state (FALSE,
			GTK_WIDGET (app)->window,
			gdk_atom_intern ("_NET_WM_STATE_FULLSCREEN", FALSE),
			GDK_NONE);
		gtk_widget_show (app->statusbar);
		gtk_widget_show (app->menubar);
	}
	
	current_status = IDLE;
	
	return TRUE;
	
}
