/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
/* $Id: lirc.c,v 1.2 2002/10/20 13:45:43 dolphy Exp $ */

#include "getcontrol.h"

#ifdef HAVE_LIRC

struct lirc_config * remote_config = NULL;

gint lirc_fd = -1, input_tag = 0;

/* execute_lirc_command
	Function analyzing the lirc order and executing corresponding order
*/

gboolean
execute_lirc_command (gchar *cmd)
{
	image_node * l_images_list = NULL;
	
	if (g_strcasecmp (cmd, LIRC_LEFT) == 0) {
		g_message ("lirc left");
		
		switch (current_status) {
			case IDLE :
			case SELECTED :
				remove_all_timers ();
				direction = LEFT;
				menu_rotation_manager (current_menu_images_list);
				return TRUE;
			case SCREEN_SAVING :
				remove_all_timers ();
				draw_menu (current_menu_images_list);
				return TRUE;
			case ROTATING :
			case DESELECTING :
				stack_order (&order_queue, LEFT, ROTATE_ORDER);
				return TRUE;
		}
			
	}
	else if (g_strcasecmp (cmd, LIRC_RIGHT) == 0) {
		g_message ("lirc right");
		
		switch (current_status) {
			case IDLE :
			case SELECTED :
				remove_all_timers ();
				direction = RIGHT;
				menu_rotation_manager (current_menu_images_list);
				return TRUE;
			case SCREEN_SAVING :
				remove_all_timers ();
				draw_menu (current_menu_images_list);
				return TRUE;
			case ROTATING :
			case DESELECTING :
				stack_order (&order_queue, RIGHT, ROTATE_ORDER);
				return TRUE;
		}
	}
	else if (g_strcasecmp (cmd, LIRC_UP) == 0) {
		g_message ("lirc up");
		return TRUE;
	}
	else if (g_strcasecmp (cmd, LIRC_DOWN) == 0) {
		g_message ("lirc down");
		return TRUE;
	}
	else if (g_strcasecmp (cmd, LIRC_FULLSCREEN) == 0) {
		g_message ("lirc fullscreen");
		
		if (fullscreen)
			fullscreen = FALSE;
		else
			fullscreen = TRUE;
					
		/* Storing the change in GConf */
	
		gconf_client_set_bool (conf_client,
			FULLSCREEN_PATH, fullscreen, NULL);
					
		return TRUE;
	}
	else if (g_strcasecmp (cmd, LIRC_VALIDATE) == 0) {
		g_message ("lirc validate");
		l_images_list = current_menu_images_list;
		while ( !(l_images_list->selected) && (l_images_list) )
			l_images_list = l_images_list->next_image;
			l_images_list->image_link (l_images_list->data);
		return TRUE;
	}
	else if (g_strcasecmp (cmd, LIRC_MAIN_MENU) == 0) {
		g_message ("lirc main menu");
		activate_main_menu (current_menu_images_list);
		return TRUE;
	}
	else
		g_message ("unrecognized lirccmd: %s\n", cmd);
	
	return TRUE;
}

gboolean
my_lirc_init (void)
{
	g_message ("Trying to bring up lirc");
	
	if ( (lirc_fd = lirc_init ("getcontrol", 0)) <= 0 ) {
		g_message ("lirc_init");
		return FALSE;
	}

	if (lirc_readconfig (NULL, &remote_config, NULL) != 0) {
		g_message ("lirc_readconfig");
		lirc_deinit ();
		lirc_fd = -1;
		return FALSE;
	}

	fcntl (lirc_fd, F_SETFL, O_NONBLOCK);
	fcntl (lirc_fd, F_SETFD, FD_CLOEXEC);

	return TRUE;
}

gboolean
my_lirc_deinit(void)
{
	
	if (lirc_fd <= 0)
		return FALSE;
	
	g_message("Shutting down lirc");
	
	gdk_input_remove (input_tag);
	
	lirc_freeconfig(remote_config);
	lirc_deinit ();
	
	input_tag = 0;
	
	return TRUE;
}

void
lirc_has_data_cb (gpointer data, gint fd, GdkInputCondition condition)
{
	gchar * code, * cmd;
	gint ret = -1;

	while (lirc_nextcode (&code) == 0 && code != NULL) {
		ret = 0;
		while (lirc_code2char (remote_config, code, &cmd) == 0 && cmd != NULL) {
			g_message ("lirc command: %s\n", cmd);
			execute_lirc_command(cmd);
		}
		if (code)
			g_free (code);
	}

	if (ret == -1) {
		
		g_message("An lirc error occured");
		gdk_input_remove (input_tag);
		lirc_freeconfig (remote_config);
		remote_config = NULL;
		lirc_deinit ();
	}
}

gboolean
start_lirc(void)
{

	if (input_tag)
		return FALSE;

	if (lirc_fd<0)
		return FALSE;

	input_tag = gdk_input_add (lirc_fd, GDK_INPUT_READ, lirc_has_data_cb, NULL);
	
	return TRUE;
}

#endif
